;
; ChangeChar v1.0 & EditString v1.0 for CrASH v1.1+, Ash v3.1+
; (C) 1998 Wouter Demuynck (kadem@unicall.be)
; http://uc2.unicall.be/kadem/ti82/
;
; Started : June 29, 1998
; Last update : August 7, 1998
;
; History:
;  June 29, 1998 - ChangeChar v0.1
;   -> First full working version, LEFT and RIGHT keys not implemented yet
;  June 29, 1998 - ChangeChar v0.11
;   -> Char now shown after a change also
;   -> Min value check implemented 
;  July 20, 1998 - ChangeChar v0.12
;   -> 2nd + '0' works now
;  August 5, 1998 - ChangeChar v0.13 - EditString v0.1
;   -> Alpha + '0' = space !
;   -> Optimizations / EditString Added
;  August 7, 1998 - ChangeChar v1.0 - EditString v1.0
;   -> Optimizations
;   -> Nicer looking source ;)
;   -> Final release (I hope so)

;
; EditString 1.0 by Wouter Demuynck (kadem@unicall.be)
; http://uc2.unicall.be/kadem/ti82/
;  Input : HL points to string
;          BC contains coordinates (B=X, C=Y)
;          A contains length of string
;  Output : String edited
; (approx. 65 bytes) 
;
EditString:
 add    a,b                         ; b = x coord, length + start x = end x
 dec    a                           ; -1 (1 too far)
 ld     (_Change_CheckLength+1),a   ; write it there
 ld     a,b                         ; b = x coord
 ld     (_Change_CheckLeft+1),a     ; start x 
 ld     (CURSOR_POS),bc ; update cursor
 push   hl 
 ROM_CALL(D_ZT_STR)     ; write string
 pop    hl
_ES_ChangeLoop:
 PUSH HL \ PUSH BC
 call   ChangeChar      ; Change character
 ld     bc,(_Change_CursorPos+1)
 inc    c               ; go to line below edited char
 ld     (CURSOR_POS),bc
 ld     c,a             ; faster than PUSH AF
 ld     a,$20           ; write a space (an arrow was written there)
 ROM_CALL(TR_CHARPUT)
 ld     a,c             ; faster than POP AF
 POP BC \ POP HL
 inc    a               ; cp $FF (enter); check result from Call ChangeChar
 ret    z               ; jr z,_ES_Done
 inc    a               ; cp $FE (right)
 jr     z,_ES_ToRight
_ES_ToLeft:             ; if not $FF or $FE, it's automatically $FD
_Change_CheckLeft:
 ld     a,0             ; will be changed
 cp     b               ; b = selected char
 jr     z,_ES_ChangeLoop
 dec    b
 dec    hl
 jr     _ES_ChangeLoop
_ES_ToRight:
_Change_CheckLength 
 ld     a,0             ; will be changed
 cp     b               ; b = selected char
 jr     z,_ES_ChangeLoop
 inc    b               ; next char selected
 inc    hl              ; same (point to next byte)
 jr     _ES_ChangeLoop
;
; End of EditString Routine
;

;
; ChangeChar v1.0 by Wouter Demuynck (kadem@unicall.be) 
;                    (http://uc2.unicall.be/kadem/ti82/)
;  input : hl=Address of char, b=col, c=row
;  output : (HL) changed 
;           A = $FF -> Enter pressed (or MODE/QUIT/CLEAR)
;               $FE -> Right key pressed
;               $FD -> Left key pressed
;           Arrow and char on screen
; Notes : 
;  Normal -> CAPS
;  ALPHA + Char -> small chars
;  ALPHA + '0' -> space
;  2ND + Number -> Number  
;  Up/Down -> Change byte (+$01 or -$01)
; (approx. 144 bytes + 62 byte letter table = 206 bytes)
;
ChangeChar:
 ld     (_Change_CursorPos+1),BC
 ld     d,h 
 ld     e,l             ; DE contains address of char to edit
 inc    c
 ld     (CURSOR_POS),bc
 ld     a,$06           ; put arrow ^
 ROM_CALL(TR_CHARPUT)
ChangeChar_Loop:
_Change_CursorPos
 ld     bc,$ABCD        ; will change
 ld     (CURSOR_POS),bc
 ld     a,(de) 
 ROM_CALL(TR_CHARPUT)   ; show char
 PUSH   DE 
 CALL   CR_KHAND        ; wait for key
 POP    DE 
 ld     c,a             ; save for later
 dec    a               ; cp K_RIGHT         ; 1
 jr     z,CC_RIGHTPRESSED
 dec    a               ; cp K_LEFT          ; 2
 jr     z,CC_LEFTPRESSED
 dec    a               ; cp K_UP            ; 3
 jr     z,CC_UP
 dec    a               ; cp K_DOWN          ; 4
 jr     z,CC_DOWN
 dec    a               ; cp K_ENTER         ; 5
 jr     z,CC_ENTERPRESSED
 dec    a               ; cp K_CLEAR         ; 6
 jr     z,CC_ENTERPRESSED
 sub    64-6            ; cp K_QUIT          ; 64
 jr     z,CC_ENTERPRESSED
 sub    70-64           ; cp K_MODE          ; 70
 jr     z,CC_ENTERPRESSED
 sub    154-70          ; cp K_SEMI          ; 154
 jr     z,CC_space
CC_CHECKCHAR:
 ld     a,c             ; get char back
 ld     hl,LetterTable
 ld     bc,26+26+10     ; 62 letters to check
 CPIR                   ; go on in mem till (hl) = A(dec bc while doing this)
 jr     nz,ChangeChar_Loop  ; nothing found -> back to loop
 ld     a,c
 cp     10
 jr     nc,CC_smallchar
CC_NUMBER:
 ld     a,'0'
 jr     CC_CHANGEIT
CC_space
 ld     a,' '
 sub    c
 jr     CC_CHANGEIT
CC_smallchar:
 sub    10
 ld     c,a
 cp     26
 jr     nc,CC_BIGCHAR
 ld     a,'a'
 jr     CC_CHANGEIT
CC_BIGCHAR:
 sub    26
 ld     c,a
 ld     a,'A'
CC_CHANGEIT:
 add    a,c     
 ld     (de),a
 ld     bc,(_Change_CursorPos+1)
 ld     (CURSOR_POS),bc
 ROM_CALL(TR_CHARPUT)   ; write new char
 ld     a,-1
CC_ENTERPRESSED:        ; A is zero when you get here
 inc    a               ; ld a,$FF
CC_RIGHTPRESSED:
 inc    a               ; ld a,$FE
CC_LEFTPRESSED:         ; (A is zero when you get here)
 add    a,$FD           ; ld a,$FD
 ret
CC_UP:                  ; (A is zero when you get here)
 ex     de,hl 
 inc    (hl)
 or     (hl)            ; is (hl) zero ?
 ex     de,hl 
 jr     nz,ChangeChar_Loop
CC_DOWN:                ; (A is zero when you get here)
 ex     de,hl 
 dec    (hl)
 cp     (hl)            ; is (hl) zero ?
 ex     de,hl 
 jr     nz,ChangeChar_Loop
 jr     CC_UP
;
; End of ChangeChar routine
;

;
; LetterTable : all keycodes for letters and chars used by ChangeChar
;
LetterTable:
 ; big letters (26)
 .db K_2,K_1,K_STO                          ;ZYX
 .db K_MINUS,K_6,K_5,K_4,K_LN               ;WVUTS
 .db K_TIMES,K_9,K_8,K_7,K_LOG              ;RQPON
 .db K_DIV,K_RPAREN,K_LPAREN,K_COMMA,K_SQR  ;MLKJI
 .db K_POWER,K_TAN,K_COS,K_SIN,K_INVERSE    ;HGFED
 .db K_PRGM,K_MATRIX,K_MATH                  ;CBA
 ; small letters (26)
 .db K_Z,K_Y,K_X,K_W,K_V,K_U,K_T,K_S
 .db K_R,K_Q,K_P,K_O,K_N,K_M,K_L,K_K
 .db K_J,K_I,K_H,K_G,K_F,K_E,K_D,K_C
 .db K_B,K_A
 ; numbers (10)                     
 .db K_n,K_Vn1,K_Un1,K_L6               ; 9876
 .db K_L5,K_L4,K_L3,K_L2,K_L1,K_0       ; 543210
