;
; Menu Subroutine v1.2 by Wouter Demuynck (kadem@unicall.be)
;   (for CrASH v1.x and Ash 3.1+ [some small changes can make it          )
;   (                             work in Ash 3.0 also, mail me if needed])
;
;  This routine allows you to make multiple menu's without having to 
; write a new routine for every menu. The menus created by this routine
; look a bit like the ones the shells use to present the programs, but
; this one also scrolls down if there are more than 9 entries.
;
;  You are permitted to use this routine in your own programs, but give me
; credit somewhere in the documentation. You are not allowed to distribute
; modified versions of this routine. If you have any ideas for making this
; routine better, please mail me at kadem@unicall.be
;
;  Take a look at MENUTEST.ASM for an example.
; --- Wouter Demuynck 
;
; Usage : add #include "MENUROUT.ASM" somewhere at the end of your program
;         and call Menu when you need a menu
; Input : HL points to menu data
; Output : A = Number of selection chosen (starting from 1)
;            = $FF if CLEAR pressed and $FE if MODE pressed
; Menu data :
;   .DB "TitleString",0            ; (16 chars max, zero terminated)
;   .db NUMBER_OF_ENTRIES    
;   .db "First selection",0
;   .db "Second selection",0       ; (all zero terminated strings)
;
; Started : May 23, 1998
; Last update : July 28, 1998 (Optimized)
;

Menu:
 ; make sure menu text goes to GRAPH_MEM
 set 7,(IY+$14)
 ; get number of entries and initiate selected choice
 ld d,(hl)           ; d contains number of entries, e will contain number
                     ; of chosen entry
 inc hl              ; HL points to title address
 ld e,1              ; to say that the first choice is selected, E=1
 ld (_Menu_Change1+1),hl ; save title addr for later use
 ld a,e              ; ld a,1
 ld (_Menu_Change2+1),a  ; first screen option selected !
 ; jump there to make hl point to first entry 
 ; I could've done ROM_CALL(D_ZT_STR) also but it takes more bytes
 jp _Menu_OneLevelDown
_Menu_ShowEntries: ; show 9 entries on screen without destroying hl
 ; Clear Graph_Mem
 push hl \ push de
 ld hl,GRAPH_MEM
 ld de,GRAPH_MEM+1
 ld bc,767
 xor a
 ld (hl),a
 ldir
 pop de \ pop hl
 push hl \ push de
 ld e,1             ; to check if we reached last entry to show (if <9)
 ld bc,$0201
 ld (CURSOR_X),bc
 ld b,9                  ; 9 entries to show
 \_MenuShow9Entries_Loop:
 ld a,(CURSOR_Y)
 add a,$06
 ld (CURSOR_Y),a
 ld a,1
 ld (CURSOR_X),a
 ROM_CALL(D_ZM_STR)          ; show entry
 ld a,e
 cp d                        ; last entry just shown ?
 jr z,_Menu_EndShowEntries
 inc e
 djnz _MenuShow9Entries_Loop
_Menu_EndShowEntries:
 pop de \ pop hl
_Menu_OK: 
 call _Menu_Invert
 ;call CR_GRBCopy    ; moved further to reduce flickering of the title
 push hl
 ld hl,0
 ld (CURSOR_POS),hl
_Menu_Change1:
 ld hl,$ABCD                     ; will change! (title addr)
 call CR_GRBCopy                 ; show menu entries with selected item
 ROM_CALL(D_ZT_STR)              ; immediately show title
 pop hl
_Menu_WaitKey:
 push hl \ push de
 call CR_KHAND
 pop de \ pop hl
 sub 3        ; cp K_UP (3)
 jr z,_Menu_UpKeyPressed
 dec a        ; cp K_DOWN (4)
 jr z,_Menu_DownKeyPressed
 cp 2         ;cp K_CLEAR (6)
 jr z,_Menu_ClearPressed
 cp 66        ;cp K_MODE (70)
 jr z,_Menu_ModePressed
 dec a       ;cp K_ENTER (5)
 jr nz,_Menu_WaitKey
 ld a,e                   ; return value of selected item in a
 ret
_Menu_VerticalChange
 jr z,_Menu_OneLevelDown
 ld (_Menu_Change2+1),a
 jr _Menu_OK
_Menu_ModePressed:
 ld a,$FE
 ret
_Menu_ClearPressed:
 ld a,$FF
 ret
_Menu_UpKeyPressed:
 ld a,e
 dec a            ; are we on first entry ?
 jr z,_Menu_WaitKey
 dec e            ; yes there is a previous choice, select it
 call _Menu_Invert
 ;ld a,(_Menu_Change2+1)
 dec a                     ; first entry on screen ?
 jr nz,_Menu_VerticalChange
_Menu_OneLevelUp:
 ;inc b                    ; b was zero after call _Menu_Invert, now bc>$100
 xor a                    ; we are looking for a zero byte
 cpdr                     ; look for it
 cpdr                     ; needs to be done twice :(
 inc hl                   ; search previous zero byte (-1)
 inc hl                   ; and go 2 up again
 jp _Menu_ShowEntries
_Menu_DownKeyPressed:
 ld a,e
 cp d             ; last entry
 jr z,_Menu_WaitKey
 inc e            ; yes there is a next choice, select is !
 call _Menu_Invert
 ;ld a,(_Menu_Change2+1)
 inc a
 cp 10          ; last entry on screen?
 jr nz,_Menu_VerticalChange
_Menu_OneLevelDown:
 ;inc b             ; bc>$100 after this
 xor a             ; look for zero byte
 cpir              ; search for next zero byte and go 1 byte further
 jp _Menu_ShowEntries
;
_Menu_Invert:
 push hl \ push de
 ld hl,GRAPH_MEM+24
 ld de,12*6
_Menu_Change2:
 ld b,1           ; will contain number of screen entry to invert
_Menu_Invert_CalcAddr:
 add hl,de
 djnz _Menu_Invert_CalcAddr
 ld b,12*7
_Menu_Invert_Loop:
 ld a,(hl)
 xor $FF
 ld (hl),a
 inc hl
 djnz _Menu_Invert_Loop
 pop de \ pop hl
 inc b  ; instead of doing this later ;) used for OneLevelUp and OneLevelDown
 ld a,(_Menu_Change2+1) ; instead of doing this later ;)
 ret
;
; End of routine
;


